'use strict';

exports.__esModule = true;

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports['default'] = reduxForm;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

function _inherits(subClass, superClass) { if (typeof superClass !== 'function' && superClass !== null) { throw new TypeError('Super expression must either be null or a function, not ' + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _actions = require('./actions');

var _util = require('./util');

function createReduxFormDecorator(sliceName, syncValidate, asyncValidate, asyncBlurFields) {
  function combineValidationErrors(form) {
    var syncErrors = syncValidate(form.data);
    var asyncErrors = _extends({ valid: true }, form.asyncErrors);
    var valid = !!(syncErrors.valid && asyncErrors.valid); // !! to convert falsy to boolean
    return _extends({}, syncErrors, asyncErrors, {
      valid: valid
    });
  }

  return function (DecoratedComponent) {
    return (function (_Component) {
      _inherits(ReduxForm, _Component);

      function ReduxForm() {
        _classCallCheck(this, ReduxForm);

        _Component.apply(this, arguments);
      }

      ReduxForm.prototype.render = function render() {
        var _this = this;

        var _props = this.props;
        var form = _props.form;
        var sliceName = _props.sliceName;
        var dispatch = _props.dispatch;

        var passableProps = _objectWithoutProperties(_props, ['form', 'sliceName', 'dispatch']);

        // eslint-disable-line no-shadow
        var runAsyncValidation = asyncValidate ? function () {
          dispatch(_actions.startAsyncValidation(sliceName));
          var promise = asyncValidate(form.data);
          if (!promise || typeof promise.then !== 'function') {
            throw new Error('asyncValidate function passed to reduxForm must return a promise!');
          }
          return promise.then(function (asyncErrors) {
            dispatch(_actions.stopAsyncValidation(sliceName, asyncErrors));
            return !!asyncErrors.valid;
          });
        } : undefined;
        var handleBlur = function handleBlur(name, value) {
          return function (event) {
            var fieldValue = value || event.target.value;
            dispatch(_actions.blur(sliceName, name, fieldValue));
            if (runAsyncValidation && ~asyncBlurFields.indexOf(name)) {
              var _extends2;

              var syncError = syncValidate(_extends({}, form.data, (_extends2 = {}, _extends2[name] = fieldValue, _extends2)))[name];
              // only dispatch async call if all synchronous client-side validation passes for this field
              if (!syncError) {
                runAsyncValidation();
              }
            }
          };
        };
        var pristine = _util.isPristine(form.initial, form.data);

        var _combineValidationErrors = combineValidationErrors(form);

        var valid = _combineValidationErrors.valid;

        var errors = _objectWithoutProperties(_combineValidationErrors, ['valid']);

        var handleChange = function handleChange(name, value) {
          return function (event) {
            return dispatch(_actions.change(sliceName, name, value || event.target.value));
          };
        };
        var handleSubmit = function handleSubmit(submitOrEvent) {
          var createEventHandler = function createEventHandler(submit) {
            return function (event) {
              if (event) {
                event.preventDefault();
              }
              dispatch(_actions.touchAll(sliceName));
              if (runAsyncValidation) {
                runAsyncValidation().then(function (asyncValid) {
                  if (valid && asyncValid) {
                    submit(form.data);
                  }
                });
              } else if (valid) {
                submit(form.data);
              }
            };
          };
          if (typeof submitOrEvent === 'function') {
            return createEventHandler(submitOrEvent);
          }
          var onSubmit = _this.props.onSubmit;

          if (!onSubmit) {
            throw new Error('You must either pass handleSubmit() an onSubmit function or pass onSubmit as a prop');
          }
          createEventHandler(onSubmit)(submitOrEvent /* is event */);
        };
        return _react2['default'].createElement(DecoratedComponent, _extends({
          asyncValidate: runAsyncValidation,
          asyncValidating: form.asyncValidating,
          data: form.data,
          dirty: !pristine,
          dispatch: dispatch,
          errors: errors,
          handleBlur: handleBlur,
          handleChange: handleChange,
          handleSubmit: handleSubmit,
          initializeForm: function (data) {
            return dispatch(_actions.initialize(sliceName, data));
          },
          invalid: !valid,
          pristine: pristine,
          resetForm: function () {
            return dispatch(_actions.reset(sliceName));
          },
          touch: function () {
            for (var _len = arguments.length, touchFields = Array(_len), _key = 0; _key < _len; _key++) {
              touchFields[_key] = arguments[_key];
            }

            return dispatch(_actions.touch.apply(undefined, [sliceName].concat(touchFields)));
          },
          touched: form.touched,
          touchAll: function () {
            return dispatch(_actions.touchAll(sliceName));
          },
          untouch: function () {
            for (var _len2 = arguments.length, untouchFields = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
              untouchFields[_key2] = arguments[_key2];
            }

            return dispatch(_actions.untouch.apply(undefined, [sliceName].concat(untouchFields)));
          },
          untouchAll: function () {
            return dispatch(_actions.untouchAll(sliceName));
          },
          valid: valid
        }, passableProps)); // pass other props
      };

      _createClass(ReduxForm, null, [{
        key: 'displayName',
        value: 'ReduxForm(' + _util.getDisplayName(DecoratedComponent) + ')',
        enumerable: true
      }, {
        key: 'DecoratedComponent',
        value: DecoratedComponent,
        enumerable: true
      }, {
        key: 'propTypes',
        value: {
          sliceName: _react.PropTypes.string,
          form: _react.PropTypes.object.isRequired,
          onSubmit: _react.PropTypes.func,
          dispatch: _react.PropTypes.func.isRequired
        },
        enumerable: true
      }, {
        key: 'defaultProps',
        value: {
          sliceName: sliceName
        },
        enumerable: true
      }]);

      return ReduxForm;
    })(_react.Component);
  };
}

function reduxForm(sliceName) {
  var syncValidate = arguments.length <= 1 || arguments[1] === undefined ? function () {
    return { valid: true };
  } : arguments[1];

  var decorator = createReduxFormDecorator(sliceName, syncValidate);
  decorator.async = function (asyncValidate) {
    for (var _len3 = arguments.length, fields = Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
      fields[_key3 - 1] = arguments[_key3];
    }

    var blurFields = Array.isArray(fields[0]) ? fields[0] : fields;
    return createReduxFormDecorator(sliceName, syncValidate, asyncValidate, blurFields);
  };
  return decorator;
}

module.exports = exports['default'];